@@ -1,8 +1,7 @@
-/*  $Id: Controller.h,v 1.8 2007/06/26 02:55:04 sarrazip Exp $
-    Controller.h - Abstract game controller
+/*  $Id: Controller.h,v 1.12 2011/10/31 01:37:53 sarrazip Exp $
 
     cosmosmash - A space rock shooting video game.
-    Copyright (C) 2007 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2007-2011 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -14,10 +13,10 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    You should have received a copy of the GNU General Public
+    License along with this program; if not, write to the Free
+    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA  02110-1301, USA.
 */
 
 #ifndef _H_Controller
@@ -42,14 +41,14 @@ public:
     virtual void startOfTick() {}
 
     /** Called by engine to inform this controller that the keyboard
-	state has changed.
+        state has changed.
     */
-    virtual void processKey(SDLKey keysym, bool pressed) {}
+    virtual void processKey(SDLKey /*keysym*/, bool /*pressed*/) {}
 
     /** Indicates if the player has just asked to start a game.
-	Returns true if yes, and then 'isExtendedMode' indicates
-	if the non-Intellivision extensions must be activated.
-	Otherwise, returns false and 'isExtendedMode' is false.
+        Returns true if yes, and then 'isExtendedMode' indicates
+        if the non-Intellivision extensions must be activated.
+        Otherwise, returns false and 'isExtendedMode' is false.
     */
     virtual bool isStartRequested(bool &isExtendedMode) = 0;
 
@@ -57,9 +56,9 @@ public:
     */
     virtual bool isEndRequested() { return false; }
 
-    virtual bool isPauseRequested() { return false; } 
+    virtual bool isPauseRequested() { return false; }
 
-    virtual bool isResumeRequested() { return false; } 
+    virtual bool isResumeRequested() { return false; }
 
     virtual bool isShootingActive() = 0;
 
@@ -69,6 +68,8 @@ public:
 
     virtual bool isRightMoveRequested() = 0;
 
+    virtual bool isFullScreenToggleRequested() = 0;
+
     /** Do any processing that needs to be done at the end of a tick.
     */
     virtual void endOfTick() {}
@@ -79,13 +80,13 @@ public:
 
     virtual void beginGameState() {}
 
-    virtual void setPlayerPos(flatzebra::Couple pos) {}
+    virtual void setPlayerPos(flatzebra::Couple /*pos*/) {}
 
-    virtual void addRockPos(flatzebra::Couple pos) {}
+    virtual void addRockPos(flatzebra::Couple /*pos*/) {}
 
-    virtual void addSpinnerPos(flatzebra::Couple pos) {}
+    virtual void addSpinnerPos(flatzebra::Couple /*pos*/) {}
 
-    virtual void addPulsarPos(flatzebra::Couple pos) {}
+    virtual void addPulsarPos(flatzebra::Couple /*pos*/) {}
 
     virtual void endGameState() {}
 
