@@ -1,8 +1,7 @@
-/*  $Id: CosmoSmashEngine.h,v 1.31 2008/11/15 03:56:36 sarrazip Exp $
-    CosmoSmashEngine.h - A space rock shooting video game engine.
+/*  $Id: CosmoSmashEngine.h,v 1.47 2017/12/09 18:20:44 sarrazip Exp $
 
     cosmosmash - A space rock shooting video game.
-    Copyright (C) 2000-2004 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2000-2011 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -14,10 +13,10 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    You should have received a copy of the GNU General Public
+    License along with this program; if not, write to the Free
+    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA  02110-1301, USA.
 */
 
 #ifndef _H_CosmoSmashEngine
@@ -29,13 +28,23 @@ class Controller;
 
 #include <flatzebra/GameEngine.h>
 #include <flatzebra/Sprite.h>
-#include <flatzebra/SoundMixer.h>
 #include <flatzebra/KeyState.h>
 
+// Sound engine:
+#include <roundbeetle/Engine.h>
+#include <roundbeetle/WaveFileBuffer.h>
+
 #include <string>
 #include <vector>
 #include <iostream>
 
+enum
+{
+    WINDOW_WIDTH = 675,
+    WINDOW_HEIGHT = 520,
+    NUM_LEVELS = 6
+};
+
 
 class CosmoSmashEngine : public flatzebra::GameEngine
 /*  A space rock shooting video game engine.
@@ -43,33 +52,40 @@ class CosmoSmashEngine : public flatzebra::GameEngine
 {
 public:
     CosmoSmashEngine(long initialScore,
-			bool mirrorHyperspace, bool useSound,
-			bool fullScreen, Controller *controller,
-			std::ostream *gameScriptStream)
-						    throw(int, std::string);
+                        bool mirrorHyperspace, bool useSound,
+                        bool fullScreen, Controller *controller,
+                        std::ostream *gameScriptStream,
+                        bool processActiveEvent);
     /*  See base class.
 
-	Throws a string or an integer code if an error occurs.
+        Throws a string or an integer code if an error occurs.
 
-	If 'mirrorHyperspace' is true, then hyperspace pivots around
-	the center of the ground.  If 'mirrorHyperspace' is false,
-	then hyperspace is random.
+        If 'mirrorHyperspace' is true, then hyperspace pivots around
+        the center of the ground.  If 'mirrorHyperspace' is false,
+        then hyperspace is random.
 
-	'controller' must be a non-null pointer to an object constructed
-	with operator new.
-	The destructor of this class will call delete on 'controller'.
+        'controller' must be a non-null pointer to an object constructed
+        with operator new.
+        The destructor of this class will call delete on 'controller'.
+
+        'processActiveEvent' determines if the game automatically pauses
+        when the game window loses focus.
     */
 
     virtual ~CosmoSmashEngine();
     /*  Destroys the Controller object given to the constructor,
-	by calling operator delete.
-	That object must have been constructed with operator new.
+        by calling operator delete.
+        That object must have been constructed with operator new.
     */
 
     virtual void processKey(SDLKey keysym, bool pressed);
     /*  Inherited.
     */
 
+    virtual void processActivation(bool appActive);
+    /*  Inherited.
+    */
+
     virtual bool tick();
     /*  Inherited.
     */
@@ -89,18 +105,70 @@ private:
 
     enum PlayerBoostType
     {
-	NO_BOOST, TRIPLE_BULLETS, CLOSER_BULLETS
+        NO_BOOST, TRIPLE_BULLETS, CLOSER_BULLETS
+    };
+
+    class SpinnerSprite : public flatzebra::Sprite
+    {
+    public:
+
+        // startFreq, endFreq: range of frequencies in Hz of spinner's
+        //                     square wave sound.
+        // durationInSeconds: life of spinner, assuming it reaches ground.
+        //
+        SpinnerSprite(const flatzebra::PixmapArray &pixmapArray,
+                      const flatzebra::Couple &pos,
+                      const flatzebra::Couple &speed,
+                      const flatzebra::Couple &collBoxPos,
+                      const flatzebra::Couple &collBoxSize,
+                      roundbeetle::Engine *mixer,
+                      float startFreq,
+                      float endFreq,
+                      float durationInSeconds)
+        :   flatzebra::Sprite(pixmapArray, pos, speed, flatzebra::Couple(0, 0),
+                              collBoxPos, collBoxSize),
+            theSoundMixer(mixer),
+            soundReq(-1),  // -1 means invalid
+            linMovFreq(startFreq, endFreq, durationInSeconds)
+        {
+        }
+
+        virtual ~SpinnerSprite()
+        {
+            if (theSoundMixer != NULL && soundReq != -1)
+                theSoundMixer->stopRequest(soundReq);
+        }
+
+        void setSoundReq(int _soundReq) { soundReq = _soundReq; }
+
+        int getSoundReq() const { return soundReq; }
+
+        roundbeetle::LinearMovingFreq &getLinearMovingFreq() { return linMovFreq; }
+
+    private:
+
+        SpinnerSprite(const SpinnerSprite &);
+        SpinnerSprite &operator = (const SpinnerSprite &);
+
+    private:
+
+        roundbeetle::Engine *theSoundMixer;  // may be null
+        int soundReq;  // handle of the RoundBeetle sound request
+        roundbeetle::LinearMovingFreq linMovFreq;  // roundbeetle thread refers to this
+
     };
 
 
-    bool paused;
+
+    enum PauseState { PLAYING, LOST_FOCUS, AWAITING_RESUME } pauseState;
     unsigned long tickCount;
     bool useSound;
 
     // SDL color values (see SDL_MapRGB()):
-    Uint32 blackColor;
+    Uint32 levelColors[NUM_LEVELS + 1];  // valid indexes: 1..NUM_LEVELS
     Uint32 whiteColor;
     Uint32 greenColor;
+    Uint32 explosionBGColor;
 
     bool mirrorHyperspace;
     bool useGameExtensions;
@@ -148,8 +216,9 @@ private:
     long   theScore;      // player's score in points
     long   thePeakScore;  // not displayed
     int    theLevel;      // level number (1--6) (depends on theScore)
-    bool   updateScore;   // indicates if player's score needs to be rewritten
     flatzebra::Couple scoreAreaPos;
+    flatzebra::Couple peakScoreAreaPos;
+    flatzebra::Couple levelNumAreaPos;
 
     int    numLives;  // number of player lives left
     bool   updateNumLives;
@@ -165,14 +234,16 @@ private:
 
     /*  SOUND EFFECTS:
     */
-    flatzebra::SoundMixer *theSoundMixer;  // see method playSoundEffect()
     enum { NUM_ROCK_HIT_SOUNDS = 2 };
-    flatzebra::SoundMixer::Chunk rockHitSounds[NUM_ROCK_HIT_SOUNDS];
-    flatzebra::SoundMixer::Chunk playerHitSound;
-    flatzebra::SoundMixer::Chunk pulsarBeepSound;
-    flatzebra::SoundMixer::Chunk saucerShootingSound;
-    flatzebra::SoundMixer::Chunk hyperspaceSound;
-    flatzebra::SoundMixer::Chunk cadenceSound;
+
+    roundbeetle::Engine *theSoundMixer;  // may be null
+    roundbeetle::WaveFileBuffer rockHitSounds[NUM_ROCK_HIT_SOUNDS];
+    roundbeetle::WaveFileBuffer playerHitSound;
+    roundbeetle::WaveFileBuffer pulsarBeepSound;
+    roundbeetle::WaveFileBuffer saucerShootingSound;
+    roundbeetle::WaveFileBuffer hyperspaceSound;
+
+    unsigned long timeBeforeNextCadenceSound;
 
 
     // Game script writing:
@@ -182,19 +253,19 @@ private:
     /*  Implementation functions:
     */
     void fireBullet(flatzebra::Couple pos);
-    void initializeSprites() throw(flatzebra::PixmapLoadError);
-    void initializeMisc(bool useSound) throw(std::string);
+    void initializeSprites();
+    void initializeMisc(bool useSound);
     void setTimeBeforeNextPulsar();
     void setTimeBeforeNextSaucer();
     void setTimeBeforeNextQuestion();
     int getSpeedUp() const;
     void readKeyboardCommands();
     flatzebra::Sprite *createExplosionSprite(flatzebra::Couple posOfCenterOfExplosion,
-				    int timeToLive);
+                                    int timeToLive);
     std::pair<flatzebra::Sprite *, flatzebra::Sprite *> splitBigRock(const flatzebra::Sprite &bigRock) const;
     void makeSpriteExplode(const flatzebra::Sprite *target,
-			    const flatzebra::Sprite *bullet,
-			    long points);
+                            const flatzebra::Sprite *bullet,
+                            long points);
     void addToScore(long n);
     void addToNumLives(int n);
     void killSpritesInList(flatzebra::SpriteList &sl);
@@ -205,24 +276,28 @@ private:
     bool playerIsActive() const;
     std::pair<bool, size_t> animateFallingObjects(flatzebra::SpriteList &sprites);
     void animateSaucers();
-    void displayPauseMessage(bool display);
+    void pause();
+    void resume();
+    void displayPauseMessage();
     void displayStartMessage(bool display);
     void displayMessage(int row, const char *msg);
     bool detectCollisionsWithBullet(const flatzebra::Sprite &bullet,
-				    flatzebra::SpriteList &slist,
-				    int scoreOnCollision);
+                                    flatzebra::SpriteList &slist,
+                                    int scoreOnCollision);
     bool detectCollisionsWithExplosions();
     void detectCollisionsBetweenFallingObjects(flatzebra::SpriteList &slist);
     void boostPlayer();
     flatzebra::Couple accelerate(flatzebra::Couple speed) const;
     void playShortExplosionSound();
-    void playSoundEffect(flatzebra::SoundMixer::Chunk &wb);
+    void playSoundEffect(const roundbeetle::WaveFileBuffer &wfb);
 
     void registerCommand(Command command);
     void registerTickEnd();
+    void playCadenceSound();
+    void startSpinnerSound(SpinnerSprite &s);
 
 
-    /*	Forbidden operations:
+    /*        Forbidden operations:
     */
     CosmoSmashEngine(const CosmoSmashEngine &);
     CosmoSmashEngine &operator = (const CosmoSmashEngine &);
